@@ -1,5 +1,3 @@
-import sys
-
 from collections import Counter
 
 import pytest
@@ -11,11 +9,6 @@ from sentry_sdk.integrations.asgi import SentryAsgiMiddleware, _looks_like_asgi3
 from async_asgi_testclient import TestClient
 
 
-minimum_python_36 = pytest.mark.skipif(
-    sys.version_info < (3, 6), reason="ASGI is only supported in Python >= 3.6"
-)
-
-
 @pytest.fixture
 def asgi3_app():
     async def app(scope, receive, send):
@@ -133,7 +126,6 @@ def asgi3_ws_app():
     return app
 
 
-@minimum_python_36
 def test_invalid_transaction_style(asgi3_app):
     with pytest.raises(ValueError) as exp:
         SentryAsgiMiddleware(asgi3_app, transaction_style="URL")
@@ -144,7 +136,6 @@ def test_invalid_transaction_style(asgi3_app):
     )
 
 
-@minimum_python_36
 @pytest.mark.asyncio
 async def test_capture_transaction(
     sentry_init,
@@ -176,7 +167,6 @@ async def test_capture_transaction(
     }
 
 
-@minimum_python_36
 @pytest.mark.asyncio
 async def test_capture_transaction_with_error(
     sentry_init,
@@ -214,7 +204,6 @@ async def test_capture_transaction_with_error(
     assert transaction_event["request"] == error_event["request"]
 
 
-@minimum_python_36
 @pytest.mark.asyncio
 async def test_has_trace_if_performance_enabled(
     sentry_init,
@@ -247,7 +236,6 @@ async def test_has_trace_if_performance_enabled(
     )
 
 
-@minimum_python_36
 @pytest.mark.asyncio
 async def test_has_trace_if_performance_disabled(
     sentry_init,
@@ -271,7 +259,6 @@ async def test_has_trace_if_performance_disabled(
     assert "trace_id" in error_event["contexts"]["trace"]
 
 
-@minimum_python_36
 @pytest.mark.asyncio
 async def test_trace_from_headers_if_performance_enabled(
     sentry_init,
@@ -305,7 +292,6 @@ async def test_trace_from_headers_if_performance_enabled(
     assert transaction_event["contexts"]["trace"]["trace_id"] == trace_id
 
 
-@minimum_python_36
 @pytest.mark.asyncio
 async def test_trace_from_headers_if_performance_disabled(
     sentry_init,
@@ -334,10 +320,9 @@ async def test_trace_from_headers_if_performance_disabled(
     assert error_event["contexts"]["trace"]["trace_id"] == trace_id
 
 
-@minimum_python_36
 @pytest.mark.asyncio
 async def test_websocket(sentry_init, asgi3_ws_app, capture_events, request):
-    sentry_init(debug=True, send_default_pii=True)
+    sentry_init(send_default_pii=True)
 
     events = capture_events()
 
@@ -367,7 +352,6 @@ async def test_websocket(sentry_init, asgi3_ws_app, capture_events, request):
     assert exc["value"] == "Oh no"
 
 
-@minimum_python_36
 @pytest.mark.asyncio
 async def test_auto_session_tracking_with_aggregates(
     sentry_init, asgi3_app, capture_envelopes
@@ -406,7 +390,6 @@ async def test_auto_session_tracking_with_aggregates(
     assert len(session_aggregates) == 1
 
 
-@minimum_python_36
 @pytest.mark.parametrize(
     "url,transaction_style,expected_transaction,expected_source",
     [
@@ -470,7 +453,6 @@ class MockAsgi3App(MockAsgi2App):
         pass
 
 
-@minimum_python_36
 def test_looks_like_asgi3(asgi3_app):
     # branch: inspect.isclass(app)
     assert _looks_like_asgi3(MockAsgi3App)
@@ -487,7 +469,6 @@ def test_looks_like_asgi3(asgi3_app):
     assert not _looks_like_asgi3(asgi2)
 
 
-@minimum_python_36
 def test_get_ip_x_forwarded_for():
     headers = [
         (b"x-forwarded-for", b"8.8.8.8"),
@@ -525,7 +506,6 @@ def test_get_ip_x_forwarded_for():
     assert ip == "5.5.5.5"
 
 
-@minimum_python_36
 def test_get_ip_x_real_ip():
     headers = [
         (b"x-real-ip", b"10.10.10.10"),
@@ -550,7 +530,6 @@ def test_get_ip_x_real_ip():
     assert ip == "8.8.8.8"
 
 
-@minimum_python_36
 def test_get_ip():
     # if now headers are provided the ip is taken from the client.
     headers = []
@@ -584,7 +563,6 @@ def test_get_ip():
     assert ip == "10.10.10.10"
 
 
-@minimum_python_36
 def test_get_headers():
     headers = [
         (b"x-real-ip", b"10.10.10.10"),
@@ -602,7 +580,6 @@ def test_get_headers():
     }
 
 
-@minimum_python_36
 @pytest.mark.asyncio
 @pytest.mark.parametrize(
     "request_url,transaction_style,expected_transaction_name,expected_transaction_source",
@@ -635,7 +612,6 @@ async def test_transaction_name(
     """
     sentry_init(
         traces_sample_rate=1.0,
-        debug=True,
     )
 
     envelopes = capture_envelopes()
@@ -654,7 +630,6 @@ async def test_transaction_name(
     )
 
 
-@minimum_python_36
 @pytest.mark.asyncio
 @pytest.mark.parametrize(
     "request_url, transaction_style,expected_transaction_name,expected_transaction_source",
@@ -698,7 +673,6 @@ async def test_transaction_name_in_traces_sampler(
     sentry_init(
         traces_sampler=dummy_traces_sampler,
         traces_sample_rate=1.0,
-        debug=True,
     )
 
     app = SentryAsgiMiddleware(asgi3_app, transaction_style=transaction_style)
