@@ -1,12 +1,17 @@
 import base64
+import sys
 import json
+import inspect
+import asyncio
 import os
+from unittest import mock
 
 import django
 import pytest
 from channels.testing import HttpCommunicator
 from sentry_sdk import capture_message
 from sentry_sdk.integrations.django import DjangoIntegration
+from sentry_sdk.integrations.django.asgi import _asgi_middleware_mixin_factory
 from tests.integrations.django.myapp.asgi import channels_application
 
 try:
@@ -14,10 +19,6 @@ try:
 except ImportError:
     from django.core.urlresolvers import reverse
 
-try:
-    from unittest import mock  # python 3.3 and above
-except ImportError:
-    import mock  # python < 3.3
 
 APPS = [channels_application]
 if django.VERSION >= (3, 0):
@@ -30,12 +31,17 @@ if django.VERSION >= (3, 0):
 @pytest.mark.asyncio
 @pytest.mark.forked
 async def test_basic(sentry_init, capture_events, application):
-    sentry_init(integrations=[DjangoIntegration()], send_default_pii=True)
+    sentry_init(
+        integrations=[DjangoIntegration()],
+        send_default_pii=True,
+    )
 
     events = capture_events()
 
     comm = HttpCommunicator(application, "GET", "/view-exc?test=query")
     response = await comm.get_response()
+    await comm.wait()
+
     assert response["status"] == 500
 
     (event,) = events
@@ -66,12 +72,17 @@ async def test_basic(sentry_init, capture_events, application):
     django.VERSION < (3, 1), reason="async views have been introduced in Django 3.1"
 )
 async def test_async_views(sentry_init, capture_events, application):
-    sentry_init(integrations=[DjangoIntegration()], send_default_pii=True)
+    sentry_init(
+        integrations=[DjangoIntegration()],
+        send_default_pii=True,
+    )
 
     events = capture_events()
 
     comm = HttpCommunicator(application, "GET", "/async_message")
     response = await comm.get_response()
+    await comm.wait()
+
     assert response["status"] == 200
 
     (event,) = events
@@ -94,7 +105,9 @@ async def test_async_views(sentry_init, capture_events, application):
     django.VERSION < (3, 1), reason="async views have been introduced in Django 3.1"
 )
 async def test_active_thread_id(sentry_init, capture_envelopes, endpoint, application):
-    with mock.patch("sentry_sdk.profiler.PROFILE_MINIMUM_SAMPLES", 0):
+    with mock.patch(
+        "sentry_sdk.profiler.transaction_profiler.PROFILE_MINIMUM_SAMPLES", 0
+    ):
         sentry_init(
             integrations=[DjangoIntegration()],
             traces_sample_rate=1.0,
@@ -105,18 +118,16 @@ async def test_active_thread_id(sentry_init, capture_envelopes, endpoint, applic
 
         comm = HttpCommunicator(application, "GET", endpoint)
         response = await comm.get_response()
-        assert response["status"] == 200, response["body"]
-
         await comm.wait()
 
-        data = json.loads(response["body"])
-
-        envelopes = [envelope for envelope in envelopes]
+        assert response["status"] == 200, response["body"]
         assert len(envelopes) == 1
 
         profiles = [item for item in envelopes[0].items if item.type == "profile"]
         assert len(profiles) == 1
 
+        data = json.loads(response["body"])
+
         for profile in profiles:
             transactions = profile.payload.json["transactions"]
             assert len(transactions) == 1
@@ -135,10 +146,17 @@ async def test_async_views_concurrent_execution(sentry_init, settings):
     settings.MIDDLEWARE = []
     asgi_application.load_middleware(is_async=True)
 
-    sentry_init(integrations=[DjangoIntegration()], send_default_pii=True)
+    sentry_init(
+        integrations=[DjangoIntegration()],
+        send_default_pii=True,
+    )
 
-    comm = HttpCommunicator(asgi_application, "GET", "/my_async_view")
-    comm2 = HttpCommunicator(asgi_application, "GET", "/my_async_view")
+    comm = HttpCommunicator(
+        asgi_application, "GET", "/my_async_view"
+    )  # sleeps for 1 second
+    comm2 = HttpCommunicator(
+        asgi_application, "GET", "/my_async_view"
+    )  # sleeps for 1 second
 
     loop = asyncio.get_event_loop()
 
@@ -154,7 +172,9 @@ async def test_async_views_concurrent_execution(sentry_init, settings):
     assert resp1.result()["status"] == 200
     assert resp2.result()["status"] == 200
 
-    assert end - start < 1.5
+    assert (
+        end - start < 2
+    )  # it takes less than 2 seconds so it was ececuting concurrently
 
 
 @pytest.mark.asyncio
@@ -173,10 +193,17 @@ async def test_async_middleware_that_is_function_concurrent_execution(
     ]
     asgi_application.load_middleware(is_async=True)
 
-    sentry_init(integrations=[DjangoIntegration()], send_default_pii=True)
+    sentry_init(
+        integrations=[DjangoIntegration()],
+        send_default_pii=True,
+    )
 
-    comm = HttpCommunicator(asgi_application, "GET", "/my_async_view")
-    comm2 = HttpCommunicator(asgi_application, "GET", "/my_async_view")
+    comm = HttpCommunicator(
+        asgi_application, "GET", "/my_async_view"
+    )  # sleeps for 1 second
+    comm2 = HttpCommunicator(
+        asgi_application, "GET", "/my_async_view"
+    )  # sleeps for 1 second
 
     loop = asyncio.get_event_loop()
 
@@ -192,7 +219,9 @@ async def test_async_middleware_that_is_function_concurrent_execution(
     assert resp1.result()["status"] == 200
     assert resp2.result()["status"] == 200
 
-    assert end - start < 1.5
+    assert (
+        end - start < 2
+    )  # it takes less than 2 seconds so it was ececuting concurrently
 
 
 @pytest.mark.asyncio
@@ -219,13 +248,13 @@ async def test_async_middleware_spans(
 
     events = capture_events()
 
-    comm = HttpCommunicator(asgi_application, "GET", "/async_message")
+    comm = HttpCommunicator(asgi_application, "GET", "/simple_async_view")
     response = await comm.get_response()
-    assert response["status"] == 200
-
     await comm.wait()
 
-    message, transaction = events
+    assert response["status"] == 200
+
+    (transaction,) = events
 
     assert (
         render_span_tree(transaction)
@@ -238,7 +267,7 @@ async def test_async_middleware_spans(
       - op="middleware.django": description="django.middleware.csrf.CsrfViewMiddleware.__acall__"
         - op="middleware.django": description="tests.integrations.django.myapp.settings.TestMiddleware.__acall__"
           - op="middleware.django": description="django.middleware.csrf.CsrfViewMiddleware.process_view"
-          - op="view.render": description="async_message"
+          - op="view.render": description="simple_async_view"
   - op="event.django": description="django.db.close_old_connections"
   - op="event.django": description="django.core.cache.close_caches"
   - op="event.django": description="django.core.handlers.base.reset_urlconf\""""
@@ -251,27 +280,25 @@ async def test_async_middleware_spans(
     django.VERSION < (3, 1), reason="async views have been introduced in Django 3.1"
 )
 async def test_has_trace_if_performance_enabled(sentry_init, capture_events):
-    sentry_init(integrations=[DjangoIntegration()], traces_sample_rate=1.0)
+    sentry_init(
+        integrations=[DjangoIntegration()],
+        traces_sample_rate=1.0,
+    )
 
     events = capture_events()
 
     comm = HttpCommunicator(asgi_application, "GET", "/view-exc-with-msg")
     response = await comm.get_response()
+    await comm.wait()
+
     assert response["status"] == 500
 
-    # ASGI Django does not create transactions per default,
-    # so we do not have a transaction_event here.
-    (msg_event, error_event) = events
-
-    assert msg_event["contexts"]["trace"]
-    assert "trace_id" in msg_event["contexts"]["trace"]
-
-    assert error_event["contexts"]["trace"]
-    assert "trace_id" in error_event["contexts"]["trace"]
+    (msg_event, error_event, transaction_event) = events
 
     assert (
         msg_event["contexts"]["trace"]["trace_id"]
         == error_event["contexts"]["trace"]["trace_id"]
+        == transaction_event["contexts"]["trace"]["trace_id"]
     )
 
 
@@ -281,12 +308,16 @@ async def test_has_trace_if_performance_enabled(sentry_init, capture_events):
     django.VERSION < (3, 1), reason="async views have been introduced in Django 3.1"
 )
 async def test_has_trace_if_performance_disabled(sentry_init, capture_events):
-    sentry_init(integrations=[DjangoIntegration()])
+    sentry_init(
+        integrations=[DjangoIntegration()],
+    )
 
     events = capture_events()
 
     comm = HttpCommunicator(asgi_application, "GET", "/view-exc-with-msg")
     response = await comm.get_response()
+    await comm.wait()
+
     assert response["status"] == 500
 
     (msg_event, error_event) = events
@@ -308,7 +339,10 @@ async def test_has_trace_if_performance_disabled(sentry_init, capture_events):
     django.VERSION < (3, 1), reason="async views have been introduced in Django 3.1"
 )
 async def test_trace_from_headers_if_performance_enabled(sentry_init, capture_events):
-    sentry_init(integrations=[DjangoIntegration()], traces_sample_rate=1.0)
+    sentry_init(
+        integrations=[DjangoIntegration()],
+        traces_sample_rate=1.0,
+    )
 
     events = capture_events()
 
@@ -322,20 +356,15 @@ async def test_trace_from_headers_if_performance_enabled(sentry_init, capture_ev
         headers=[(b"sentry-trace", sentry_trace_header.encode())],
     )
     response = await comm.get_response()
+    await comm.wait()
+
     assert response["status"] == 500
 
-    # ASGI Django does not create transactions per default,
-    # so we do not have a transaction_event here.
-    (msg_event, error_event) = events
-
-    assert msg_event["contexts"]["trace"]
-    assert "trace_id" in msg_event["contexts"]["trace"]
-
-    assert error_event["contexts"]["trace"]
-    assert "trace_id" in error_event["contexts"]["trace"]
+    (msg_event, error_event, transaction_event) = events
 
     assert msg_event["contexts"]["trace"]["trace_id"] == trace_id
     assert error_event["contexts"]["trace"]["trace_id"] == trace_id
+    assert transaction_event["contexts"]["trace"]["trace_id"] == trace_id
 
 
 @pytest.mark.asyncio
@@ -344,7 +373,9 @@ async def test_trace_from_headers_if_performance_enabled(sentry_init, capture_ev
     django.VERSION < (3, 1), reason="async views have been introduced in Django 3.1"
 )
 async def test_trace_from_headers_if_performance_disabled(sentry_init, capture_events):
-    sentry_init(integrations=[DjangoIntegration()])
+    sentry_init(
+        integrations=[DjangoIntegration()],
+    )
 
     events = capture_events()
 
@@ -358,16 +389,12 @@ async def test_trace_from_headers_if_performance_disabled(sentry_init, capture_e
         headers=[(b"sentry-trace", sentry_trace_header.encode())],
     )
     response = await comm.get_response()
+    await comm.wait()
+
     assert response["status"] == 500
 
     (msg_event, error_event) = events
 
-    assert msg_event["contexts"]["trace"]
-    assert "trace_id" in msg_event["contexts"]["trace"]
-
-    assert error_event["contexts"]["trace"]
-    assert "trace_id" in error_event["contexts"]["trace"]
-
     assert msg_event["contexts"]["trace"]["trace_id"] == trace_id
     assert error_event["contexts"]["trace"]["trace_id"] == trace_id
 
@@ -407,7 +434,7 @@ BODY_FORM_CONTENT_LENGTH = str(len(BODY_FORM)).encode("utf-8")
             [(b"content-type", b"application/json")],
             "post_echo_async",
             b'{"username":"xyz","password":"xyz"}',
-            {"username": "xyz", "password": "xyz"},
+            {"username": "xyz", "password": "[Filtered]"},
         ),
         (
             True,
@@ -426,7 +453,7 @@ BODY_FORM_CONTENT_LENGTH = str(len(BODY_FORM)).encode("utf-8")
             ],
             "post_echo_async",
             BODY_FORM,
-            {"password": "hello123", "photo": "", "username": "Jane"},
+            {"password": "[Filtered]", "photo": "", "username": "Jane"},
         ),
         (
             False,
@@ -490,10 +517,8 @@ async def test_asgi_request_body(
     expected_data,
 ):
     sentry_init(
+        integrations=[DjangoIntegration()],
         send_default_pii=send_default_pii,
-        integrations=[
-            DjangoIntegration(),
-        ],
     )
 
     envelopes = capture_envelopes()
@@ -506,9 +531,9 @@ async def test_asgi_request_body(
         body=body,
     )
     response = await comm.get_response()
-    assert response["status"] == 200
-
     await comm.wait()
+
+    assert response["status"] == 200
     assert response["body"] == body
 
     (envelope,) = envelopes
@@ -518,3 +543,151 @@ async def test_asgi_request_body(
         assert event["request"]["data"] == expected_data
     else:
         assert "data" not in event["request"]
+
+
+@pytest.mark.asyncio
+@pytest.mark.skipif(
+    sys.version_info >= (3, 12),
+    reason=(
+        "asyncio.iscoroutinefunction has been replaced in 3.12 by inspect.iscoroutinefunction"
+    ),
+)
+async def test_asgi_mixin_iscoroutinefunction_before_3_12():
+    sentry_asgi_mixin = _asgi_middleware_mixin_factory(lambda: None)
+
+    async def get_response(): ...
+
+    instance = sentry_asgi_mixin(get_response)
+    assert asyncio.iscoroutinefunction(instance)
+
+
+@pytest.mark.skipif(
+    sys.version_info >= (3, 12),
+    reason=(
+        "asyncio.iscoroutinefunction has been replaced in 3.12 by inspect.iscoroutinefunction"
+    ),
+)
+def test_asgi_mixin_iscoroutinefunction_when_not_async_before_3_12():
+    sentry_asgi_mixin = _asgi_middleware_mixin_factory(lambda: None)
+
+    def get_response(): ...
+
+    instance = sentry_asgi_mixin(get_response)
+    assert not asyncio.iscoroutinefunction(instance)
+
+
+@pytest.mark.asyncio
+@pytest.mark.skipif(
+    sys.version_info < (3, 12),
+    reason=(
+        "asyncio.iscoroutinefunction has been replaced in 3.12 by inspect.iscoroutinefunction"
+    ),
+)
+async def test_asgi_mixin_iscoroutinefunction_after_3_12():
+    sentry_asgi_mixin = _asgi_middleware_mixin_factory(lambda: None)
+
+    async def get_response(): ...
+
+    instance = sentry_asgi_mixin(get_response)
+    assert inspect.iscoroutinefunction(instance)
+
+
+@pytest.mark.skipif(
+    sys.version_info < (3, 12),
+    reason=(
+        "asyncio.iscoroutinefunction has been replaced in 3.12 by inspect.iscoroutinefunction"
+    ),
+)
+def test_asgi_mixin_iscoroutinefunction_when_not_async_after_3_12():
+    sentry_asgi_mixin = _asgi_middleware_mixin_factory(lambda: None)
+
+    def get_response(): ...
+
+    instance = sentry_asgi_mixin(get_response)
+    assert not inspect.iscoroutinefunction(instance)
+
+
+@pytest.mark.parametrize("application", APPS)
+@pytest.mark.asyncio
+async def test_async_view(sentry_init, capture_events, application):
+    sentry_init(
+        integrations=[DjangoIntegration()],
+        traces_sample_rate=1.0,
+    )
+
+    events = capture_events()
+
+    comm = HttpCommunicator(application, "GET", "/simple_async_view")
+    await comm.get_response()
+    await comm.wait()
+
+    (event,) = events
+    assert event["type"] == "transaction"
+    assert event["transaction"] == "/simple_async_view"
+
+
+@pytest.mark.parametrize("application", APPS)
+@pytest.mark.asyncio
+async def test_transaction_http_method_default(
+    sentry_init, capture_events, application
+):
+    """
+    By default OPTIONS and HEAD requests do not create a transaction.
+    """
+    sentry_init(
+        integrations=[DjangoIntegration()],
+        traces_sample_rate=1.0,
+    )
+    events = capture_events()
+
+    comm = HttpCommunicator(application, "GET", "/simple_async_view")
+    await comm.get_response()
+    await comm.wait()
+
+    comm = HttpCommunicator(application, "OPTIONS", "/simple_async_view")
+    await comm.get_response()
+    await comm.wait()
+
+    comm = HttpCommunicator(application, "HEAD", "/simple_async_view")
+    await comm.get_response()
+    await comm.wait()
+
+    (event,) = events
+
+    assert len(events) == 1
+    assert event["request"]["method"] == "GET"
+
+
+@pytest.mark.parametrize("application", APPS)
+@pytest.mark.asyncio
+async def test_transaction_http_method_custom(sentry_init, capture_events, application):
+    sentry_init(
+        integrations=[
+            DjangoIntegration(
+                http_methods_to_capture=(
+                    "OPTIONS",
+                    "head",
+                ),  # capitalization does not matter
+            )
+        ],
+        traces_sample_rate=1.0,
+    )
+    events = capture_events()
+
+    comm = HttpCommunicator(application, "GET", "/simple_async_view")
+    await comm.get_response()
+    await comm.wait()
+
+    comm = HttpCommunicator(application, "OPTIONS", "/simple_async_view")
+    await comm.get_response()
+    await comm.wait()
+
+    comm = HttpCommunicator(application, "HEAD", "/simple_async_view")
+    await comm.get_response()
+    await comm.wait()
+
+    assert len(events) == 2
+
+    (event1, event2) = events
+    assert event1["request"]["method"] == "OPTIONS"
+    assert event2["request"]["method"] == "HEAD"
